from collections import Counter
import numpy as np

class BagOfVectors(object):
  '''
  Some helper functions to read and update word vectors
  '''
  def __init__(self, train_data, max_vocab=10000, dim=100):
    self.train_data = train_data
    self.max_vocab = max_vocab
    self.dim = dim

    # the 0th index is for unknown words
    self.word_vectors = np.random.randn(self.max_vocab+1, self.dim) / np.sqrt(self.dim)
    counts = Counter({})
    for sent in train_data:
      for w in sent:
        counts[w] += 1
    vocabs = counts.most_common(max_vocab)

    print('top 10 most frequent tokens', vocabs[:10])

    self.index = Counter({})
    for ind, w in enumerate(vocabs):
      self.index[w[0]] = ind + 1

  def indices(self, sent):
    return [self.index[w] for w in sent]

  def vector(self, sent):
    inds = self.indices(sent)
    return np.mean(self.word_vectors[inds, :], 0)

  def data_matrix(self, data):
    X = np.zeros((len(data), self.dim))
    for ind, sent in enumerate(data):
      X[ind, :] = self.vector(sent)
    return X

  def update(self, sent, grad):
    pass
    # TODO: implement this update given some gradient
